GetBitOps

Trait GetBitOps 

pub trait GetBitOps
where Self: GetSimd + 'static,
{ // Required method fn bit_ops<I: Isa>(isa: I) -> impl BitOps<Self, Simd = Self::Simd<I>>; }
Get the BitOps implementation from an Isa for a given element type.

This is the bit-level counterpart of GetNumOps. It is implemented for all element types which support bit-level SIMD operations, which is a superset of the types that support arithmetic operations.

Required Methods

fn bit_ops<I: Isa>(isa: I) -> impl BitOps<Self, Simd = Self::Simd<I>>

Return the BitOps implementation from a SIMD Isa that provides operations on vectors containing elements of type Self.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementations on Foreign Types

Source

impl GetBitOps for f32

Source

fn bit_ops<I: Isa>( isa: I, ) -> impl BitOps<Self, Simd = <Self as GetSimd>::Simd<I>>

Source

impl GetBitOps for i8

Source

fn bit_ops<I: Isa>( isa: I, ) -> impl BitOps<Self, Simd = <Self as GetSimd>::Simd<I>>

Source

impl GetBitOps for i16

Source

fn bit_ops<I: Isa>( isa: I, ) -> impl BitOps<Self, Simd = <Self as GetSimd>::Simd<I>>

Source

impl GetBitOps for i32

Source

fn bit_ops<I: Isa>( isa: I, ) -> impl BitOps<Self, Simd = <Self as GetSimd>::Simd<I>>

Source

impl GetBitOps for u8

Source

fn bit_ops<I: Isa>( isa: I, ) -> impl BitOps<Self, Simd = <Self as GetSimd>::Simd<I>>

Source

impl GetBitOps for u16

Source

fn bit_ops<I: Isa>( isa: I, ) -> impl BitOps<Self, Simd = <Self as GetSimd>::Simd<I>>

Implementors