MaskOps

Trait MaskOps 

pub unsafe trait MaskOps<M: Mask>: Copy {
    // Required methods
    fn and(self, x: M, y: M) -> M;
    fn any(self, x: M) -> bool;
    fn all(self, x: M) -> bool;

    // Provided method
    fn all_false(self, x: M) -> bool { ... }
}
SIMD operations on a Mask vector.

Safety

Implementations must ensure they can only be constructed if the instruction set is supported on the current system.

Required Methods

fn and(self, x: M, y: M) -> M

Compute x & y.

fn any(self, x: M) -> bool

Return true if any lanes are true.

fn all(self, x: M) -> bool

Return true if all lanes are true.

Provided Methods

fn all_false(self, x: M) -> bool

Return true if all lanes are false.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors